@@ -28,9 +28,9 @@ class TestView(unittest.TestCase):
 
     def test_default_output_encoding(self):
         info = dict(someval="someval")
-        # default encoding is utf8
+        # default encoding is utf-8
         val = view.render(info, template="turbogears.tests.simple")
-        assert 'utf8' in view.cherrypy.response.headers["Content-Type"]
+        assert 'utf-8' in view.cherrypy.response.headers["Content-Type"]
         config.update({'tg.defaultview':'kid', 'kid.encoding':'iso-8859-1'})
         val = view.render(info, template="turbogears.tests.simple")
         assert 'iso-8859-1' in view.cherrypy.response.headers["Content-Type"]
