@@ -172,9 +172,9 @@ class CatWalk(turbogears.controllers.Controller):
             except:
                 return {'column':column['columnName'],'value':'None','id':'0'}
             return {'column':column['columnName'],'value':value }           
-        elif column.get('type','') == 'SORelatedJoin':
+        elif column.get('type','') in ('SORelatedJoin', 'SOSQLRelatedJoin'):
             return self.related_join_count(row,column)
-        elif column.get('type','') == 'SOMultipleJoin':
+        elif column.get('type','') in ('SOMultipleJoin', 'SOSQLMultipleJoin'):
             return self.multiple_join_count(row,column)
         elif column.get('type','') == 'SOForeignKey':
             return self.object_field_for_foreign_key(row,column)
@@ -208,7 +208,7 @@ class CatWalk(turbogears.controllers.Controller):
     def related_join_count(self,row,column):
         """Return the total number of related objects"""
         try:
-            value='%s'% len(getattr(row,column['joinMethodName']))
+            value='%s'% len(list(getattr(row,column['joinMethodName'])))
         except:
             value='0'
         return { 'column':column['joinMethodName'], 'value':value }
@@ -405,7 +405,7 @@ class CatWalk(turbogears.controllers.Controller):
         hostObject = objectName
         obj = self.load_object(objectName)
         try:
-            rows = getattr(obj.get(id),join)
+            rows = list(getattr(obj.get(id),join))
         except:
             msg='Error, joins objectName: %s, id: %s, join: %s'% (objectName,id,join)
             raise cherrypy.HTTPRedirect(turbogears.url('error', msg=msg))
@@ -487,7 +487,8 @@ class CatWalk(turbogears.controllers.Controller):
 
         if props['type'] == 'SOEnumCol': 
             props['options'] = column.enumValues
-        if props['type'] in ('SOMultipleJoin','SORelatedJoin'):
+        if props['type'] in ('SOMultipleJoin', 'SOSQLMultipleJoin',
+          'SORelatedJoin', 'SOSQLRelatedJoin'):
             props['join'] = column.otherClassName
             props['joinMethodName'] = column.joinMethodName
         if props['type'] == 'SOSingleJoin':
@@ -847,19 +848,11 @@ class CatWalk(turbogears.controllers.Controller):
            @type relatedObjectName: string
         """
         joins = self.object_joins(objectName,id,join,'SORelatedJoin',relatedObjectName)
+
         joins['allJoins'] = self.object_instances(relatedObjectName)
         return joins
     manageRelatedJoins = turbogears.expose(format="json")(manageRelatedJoins)
-
-    def uppercase_related_join_method(self,name,method):
-        """SQLObject automatically camelcase related joins methods, so we have to
-           be shure that even if the users model class is lowercase it's join methods
-           still should be camelcased
-        """
-        name = name[0].upper() + name[1:]
-        return '%s%s'% (method,name)
-
-    
+   
     def updateJoins(self,objectName,id,join,joinType,joinObjectName,joins):
         """Drop all related joins first, then loop trought the submited joins
            and set the relation
@@ -880,19 +873,24 @@ class CatWalk(turbogears.controllers.Controller):
         try:
             obj = self.load_object(objectName)
             inst = obj.get(id)
-            #drop all related joins
-            for j in list(getattr(inst,join)):
-                method = self.uppercase_related_join_method(joinObjectName,'remove') 
-                getattr(inst,method)(j)
+            
+            # get the add/remove method names
+            j = [j for j in obj.sqlmeta.joins if (j.joinMethodName == join)][0]
+            remove_method = getattr(inst, 'remove' +  j.addRemoveName)
+ 	    add_method = getattr(inst, 'add' + j.addRemoveName)
+ 	    
+            # remove all joined instances
+            for joined_instance in list(getattr(inst, join)):
+                remove_method(joined_instance)
 
-            #add the new joins
-            joinObject = self.load_object(joinObjectName)
+            # add the new joined instances
+            join_object = self.load_object(joinObjectName)
             joins = joins.split(',')
-            for j in joins:
-                try:int(j) 
-                except:continue
-                method = self.uppercase_related_join_method(joinObjectName,'add')
-                getattr(inst,method)(joinObject.get(j))
+            for i in joins:
+                try: i=int(i) 
+                except: continue
+                instance = join_object.get(i)
+                add_method(instance)
         except Exception, e:
             cherrypy.response.status = 500
             return dict(error=str(e))
@@ -963,10 +961,11 @@ class CatWalk(turbogears.controllers.Controller):
             colForID=col.copy()
             colForID['type']=''
             col['object_id']=id
-            if col['type'] in ('SOMultipleJoin','SORelatedJoin'):
+            if col['type'] in ('SOMultipleJoin', 'SOSQLMultipleJoin',
+              'SORelatedJoin', 'SOSQLRelatedJoin'):
                 col['id'] = id
                 try:
-                    value = '%s'% len(getattr(inst,col['columnName']))
+                    value = '%s'% len(list(getattr(inst,col['columnName'])))
                 except:
                     value = '0'
                 col['value'] = {'value':value, 'column':col['columnName']}
