@@ -129,6 +129,10 @@ def build_name_from_path(path, repeating_marker='-', nesting_marker='.'):
             name.append(item.widget.name)
     return nesting_marker.join(name)
 
+###############################################################################
+# Base class for a widget that can generate input fort the app.
+###############################################################################
+
 class InputWidget(Widget):
     validator = None
     params = ["convert"]
@@ -137,6 +141,18 @@ class InputWidget(Widget):
     convert = True
 
     def __init__(self, name=None, validator=None, **params):
+        """Initializes an input widget.
+
+        It accepts the following parameters (besides those listed at params):
+
+          name: 
+             Name of the input element. Will be the name of the variable
+             the widget's input will be available at when submitted.
+
+          validator: 
+             Formencode validator that will validate and coerce the input
+             this widget generates.
+        """
         if name is not None and ('-' in name or '.' in name):
             raise ValueError, ("The name of an InputWidget must not contain "
                                "the '-' or '.' characters")
@@ -243,14 +259,23 @@ class CompoundInputWidget(CompoundWidget, InputWidget):
         params['error_for'] = lambda f: self.error_for(f, True)
 
     def value_for(self, item, value):
+        """Return the value for a child widget. ``item`` is the child
+        widget instance or it's name, ``value`` is a dict containing the
+        value for this compound widget."""
         path = path_from_item(item)
         return retrieve_value_by_path(value, path) 
 
     def params_for(self, item, **params):
+        """Return the parameters for a child widget. ``item`` is the child
+        widget instance or it's name, ``params`` is a dict containing the
+        parameters passed to this compound widget."""
         path = path_from_item(item)
         return retrieve_params_by_path(params, path)
 
     def error_for(self, item, suppress_errors=False):
+        """Return's the Invalid exception associated to a child widget from 
+        request local storage. ``item`` is the child widget instance or it's
+        name."""
         if self.is_validated:
             path = path_from_item(item, self.name_path)
             errors = getattr(request, "validation_errors", {})
@@ -264,6 +289,10 @@ class CompoundInputWidget(CompoundWidget, InputWidget):
             return None
 
     def dictify_value(self, value):
+        """Converts a value into a dict suitable for propagating values to
+        child widgets. If value is a dict it will pass thorugh, if it other
+        kind of object, attributes which match child widgets' names will be 
+        tried to fetch."""
         if isinstance(value, dict):
             return value
         else:
@@ -276,6 +305,7 @@ class CompoundInputWidget(CompoundWidget, InputWidget):
             return value_as_dict
 
     def adjust_value(self, value=None, **params):
+        """Adjust a value for displaying in a widget."""
         if value is not None:
             value = self.dictify_value(value)
         return super(CompoundWidget, self).adjust_value(value, **params)
@@ -300,6 +330,7 @@ class RepeatingRange(object):
             return value
 
 class RepeatingInputWidget(CompoundInputWidget):
+    """Base class for a compound widget which can be repeated."""
     repeating = True
     params = ["repetitions"]
     params_doc = {'repetitions': 'Number of repetitions that should be '
@@ -321,6 +352,9 @@ class RepeatingInputWidget(CompoundInputWidget):
         d["repetitions"] = RepeatingRange(repetitions, path_reference)
 
     def value_for(self, item, value):
+        """Return the value for a child widget. ``item`` is the child
+        widget instance or it's name, ``value`` is a dict containing the
+        value for this compound widget."""
         if isinstance(value, list):
             try:
                 value = value[self.path[-1].repetition]
@@ -332,6 +366,9 @@ class RepeatingInputWidget(CompoundInputWidget):
         return retrieve_value_by_path(value, path) 
 
     def params_for(self, item, **params):
+        """Return the parameters for a child widget. ``item`` is the child
+        widget instance or it's name, ``params`` is a dict containing the
+        parameters passed to this compound widget."""
         item_params = {}
         for k,v in params.iteritems():
             if isinstance(v, list) and k is not "repetitions":
@@ -343,6 +380,11 @@ class RepeatingInputWidget(CompoundInputWidget):
         return retrieve_params_by_path(item_params, path)
     
     def dictify_value(self, value):
+        """Converts a list of values into a list of dicts suitable for 
+        propagating values to child widgets. If value is a list of dicts it 
+        will pass thorugh, if it other kind of object, attributes which match 
+        child widgets' names will be
+        tried to fetch."""
         return [super(RepeatingInputWidget, self).dictify_value(v)
                 for v in value]
 
@@ -350,6 +392,17 @@ class RepeatingInputWidget(CompoundInputWidget):
 # Base classes                                                              #
 #############################################################################
 class FormField(InputWidget):
+    """An input widget that can be included inside a Form or Fieldset.
+
+    It accepts the following parameters (besides those listed at params):
+
+    label
+       The label text that the container form/fieldset will generate for the 
+       widget. If empty, the capitalized name will be used.
+    help_text
+       The help text that the container form/fieldset will generate for the 
+       widget.
+    """
     _name = "widget"
     label = None
     help_text = None
@@ -424,6 +477,29 @@ def retrieve_field_for(func):
     return _retrieve_field_for
 
 class FormFieldsContainer(CompoundInputWidget):
+    """A container for FormFields.
+
+    Has two member_widgets lists:
+       - fields
+       - hidden_fields
+
+    It provides the template with 3 useful functions:
+       - field_for(name)
+           Returns the child named ``name``
+       - value_for(name)
+           Returns the value for the child named ``name`` (name can also be
+           a widget instance so fields can be iterated)
+       - params_for(name)
+           Returns the display-time parameters for the child named ``name`` (can
+           also be a widget instance so fields can be iterated)
+       - display_field_for(name)
+           displays the child named ``name`` automatically propagating value
+           an params. ``name`` can also be a widget instance.
+       - error_for(name)
+           Returns the validation error the child named ``name`` generated.
+           Again, ``name`` can also be a widget instance.
+    """
+        
     member_widgets = ["fields", "hidden_fields"]
     fields = []
     hidden_fields = []
@@ -693,7 +769,8 @@ class CheckBox(FormField):
 
     def __init__(self, *args, **kw):
         super(CheckBox, self).__init__(*args, **kw)
-        self.validator = validators.Bool()
+        if not self.validator:
+            self.validator = validators.Bool()
 
     def update_params(self, d):
         super(CheckBox, self).update_params(d)
@@ -817,7 +894,7 @@ class SelectionField(FormField):
         for optgroup in d["options"]:
             if isinstance(optgroup[1], list):
                 group = True
-                optlist = optgroup[1]
+                optlist = optgroup[1][:]
             else:
                 group = False
                 optlist = [optgroup]
@@ -825,7 +902,7 @@ class SelectionField(FormField):
                 if len(option) is 2:
                     option_attrs = {}
                 elif len(option) is 3:
-                    option_attrs = option[2]
+                    option_attrs = dict(option[2])
                 if self._is_option_selected(option[0], d['value']):
                         option_attrs[self._selected_verb] = self._selected_verb
                 optlist[i] = (option[0], option[1], option_attrs)
@@ -1089,16 +1166,6 @@ class Form(FormFieldsContainer):
         if self.validator:
             return self.validator.to_python(value, state)
 
-    def field_for(self, item):
-        import warnings
-        warnings.warn(
-            "Use of field_for() is deprecated and highly discouraged, "
-            "use display_field_for() or render_field_for() instead. "
-            "Note: this warning will be removed once 1.0 is "
-            "released and your actual code will stop working.",            
-            DeprecationWarning, 2)
-        return self.get_field_by_name(item, "Field for '%s' not found." % item)
-
     def _get_errors(self):
         return getattr(request, "validation_errors", None)
     errors = property(_get_errors)
